"use strict";
var __importDefault = (this && this.__importDefault) || function (mod) {
    return (mod && mod.__esModule) ? mod : { "default": mod };
};
Object.defineProperty(exports, "__esModule", { value: true });
exports.DynamoDB = void 0;
const aws_sdk_1 = __importDefault(require("aws-sdk"));
const object_pick_1 = __importDefault(require("object.pick"));
const table_1 = require("./table");
const methods_1 = require("./methods");
const utils_1 = require("./utils");
class DynamoDB {
    constructor() {
        this.options = {};
    }
    connect(options) {
        this.options = Object.assign({ prefix: '', prefixDelimiter: '.', host: 'localhost', localPort: 8000 }, options);
        this._retries = utils_1.configureRetryOptions(this.options.retries);
        aws_sdk_1.default.config.update(object_pick_1.default(this.options, ['region', 'accessKeyId', 'secretAccessKey', 'sessionToken']));
        if (this.options.local) {
            // Starts dynamodb in local mode
            this.raw = new aws_sdk_1.default.DynamoDB({
                endpoint: `http://${this.options.host}:${this.options.localPort}`
            });
        }
        else {
            // Starts dynamodb in remote mode
            this.raw = new aws_sdk_1.default.DynamoDB();
        }
        this.dynamodb = new aws_sdk_1.default.DynamoDB.DocumentClient({
            service: this.raw
        });
    }
    get delimiter() {
        return this.options.prefixDelimiter;
    }
    get prefix() {
        return this.options.prefix;
    }
    get retries() {
        return this._retries;
    }
    /**
     * Returns the table that can be used to interact with it.
     *
     * @param  name		The name of the table that is being interacted with.
     * @param  options	Options object.
     */
    table(name, options) {
        return new table_1.Table(name, this, options);
    }
    /**
     * Returns the table that can be used to interact with. The table name will not be prefixed automatically.
     *
     * @param  name		The name of the table that is being interacted with.
     */
    rawTable(name) {
        return new table_1.Table(name, this, { raw: true });
    }
    /**
     * Instantiate a dropped table object.
     *
     * @param	name		The name of the table that should be dropped.
     * @param	options		Options object.
     */
    dropTable(name, options) {
        return this.table(name, options).drop();
    }
    /**
     * Instantiate a raw dropped table object. The table name will not be prefixed automatically.
     *
     * @param	name		The name of the table that should be dropped.
     */
    dropRawTable(name) {
        return this.dropTable(name, { raw: true });
    }
    /**
     * Instantiate a create table object.
     *
     * @param	schema		The schema of the table that should be created.
     * @param	options		Options object.
     */
    createTable(schema, options) {
        if (typeof schema !== 'object') {
            throw new TypeError(`Expected \`schema\` to be of type \`object\`, got \`${typeof schema}\``);
        }
        if (!schema.TableName) {
            throw new Error('Schema is missing a `TableName`');
        }
        return this.table(schema.TableName, options).create(schema);
    }
    /**
     * Instantiate a create table object.
     *
     * @param	schema		The schema of the table that should be created.
     */
    createRawTable(schema) {
        return this.createTable(schema, { raw: true });
    }
    /**
     * Instantiate a list tables object.
     */
    listTables() {
        return new methods_1.ListTables(this);
    }
    /**
     * Start a write transaction with the provided actions.
     *
     * @param	actions		List of transaction actions.
     */
    transactWrite(...actions) {
        return new methods_1.TransactWrite(this, actions);
    }
    /**
     * Start a read transaction with the provided actions.
     *
     * @param	actions		List of transaction actions.
     */
    transactRead(...actions) {
        return new methods_1.TransactRead(this, actions);
    }
}
exports.DynamoDB = DynamoDB;
//# sourceMappingURL=dynamodb.js.map